﻿// See https://aka.ms/new-console-template for more information

using System.Net.Http.Headers;
using System.Text;
using System.Text.Json.Nodes;

namespace ConsoleApp1
{
    internal class NeoEdgeUser
    {
        public int id;
        public string account = string.Empty;
        public string name;
        public string roleName;
        public int roleId;
        public int accountStatus;
        public int isMfaEnable;
        public List<string> cidrWhiteList = new List<string>();
        public List<string> projectList = new List<string>();
    }
    public class Program
    {
        public static string _origin = "fqdn.neoedgecentral.com";                               // The FQDN assign to your company 
        public static string _apiBase = "https://api.neoedgecentral.com";                       // Restful API URL;
        public static string _adminEmail = "email@xxx.com";                                     // login email for API user account
        public static string _adminPassword = "password";                                      // password for API user account
        public static HttpClient _client = new HttpClient();
        public static string _accessToken = string.Empty;

        public static Dictionary<string, int> _roleList = new Dictionary<string, int>();        // Role List of your company
        public static Dictionary<string, int> _projectList = new Dictionary<string, int>();     // Project List of your company

        // Login to NeoEdge Central
        private static void loginNeoEdgeCentral(string userEmail, string password)
        {
            string fullURL = _apiBase + "/api/login";
            _client.DefaultRequestHeaders.Add("Origin", _origin);
            _client.BaseAddress = new Uri(fullURL);
            HttpRequestMessage request = new HttpRequestMessage(HttpMethod.Post, fullURL);
            request.Content = new StringContent("{\"account\":\"" + userEmail + "\",\"password\":\"" + password + "\"}",Encoding.UTF8,"application/json");
            var response = _client.Send(request);
            using var reader = new StreamReader(response.Content.ReadAsStream());
            var objects = JsonNode.Parse(reader.ReadToEnd());
            _accessToken = (string)objects!["accessToken"];
            Console.WriteLine("accessToken: " + _accessToken);
        }
        // Read all Role (Name, ID)
        private static void retrieveRoleList()
        {
            var data = callAPI("/api/v2/roles?page=1&size=10", "GET", string.Empty);

            var objects = JsonNode.Parse(data);
            for (int i = 0; i < (int)objects!["total"]; i++)
            {
                _roleList.Add((string)objects!["roles"][i]["name"], (int)objects!["roles"][i]["id"]);
            }
        }
        // Read all Project (Name, ID)
        private static void retrieveProjectList()
        {
            var data = callAPI("/api/v2/projects?page=1&size=10", "GET", string.Empty);

            var objects = JsonNode.Parse(data);
            for (int i = 0; i < (int)objects!["total"]; i++)
            {
                _projectList.Add((string)objects!["projects"][i]["name"], (int)objects!["projects"][i]["id"]);
            }
        }
        // Find User Entity by Email
        private static NeoEdgeUser findUserByEmail(string email)
        {
            var user = new NeoEdgeUser();
            var data = callAPI("/api/v2/users?page=1&size=10&accounts=" + email, "GET", string.Empty);

            var objects = JsonNode.Parse(data);
            if ((int)objects!["total"] == 1)
            {
                user.id = (int)objects!["users"][0]["id"];
                user.account = (string)objects!["users"][0]["account"];
                user.name = (string)objects!["users"][0]["name"];
                user.roleName = (string)objects!["users"][0]["roleName"];
                user.roleId = _roleList[user.roleName];
                user.accountStatus = (int)objects!["users"][0]["accountStatus"];
                user.isMfaEnable = (int)objects!["users"][0]["isMfaEnable"];
                var projects = (JsonArray)objects!["users"][0]["projects"];
                if (projects != null)
                {
                    foreach (var project in projects) {
                        user.projectList.Add((string)project);
                    }
                }
            }
            return user;
        }
        // Function
        private static string callAPI(string url, string method, string data)
        {
            string fullURL = _apiBase + url;
            _client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", _accessToken);
            HttpRequestMessage request = new HttpRequestMessage();
            switch (method.ToUpper())
            {
                case "GET":
                    request = new HttpRequestMessage(HttpMethod.Get, fullURL);
                    break;
                case "POST":
                    request = new HttpRequestMessage(HttpMethod.Post, fullURL);
                    break;
                case "PUT":
                    request = new HttpRequestMessage(HttpMethod.Put, fullURL);
                    break;
                case "DELETE":
                    request = new HttpRequestMessage(HttpMethod.Delete, fullURL);
                    break;
            }
            request.Content = new StringContent(data, Encoding.UTF8, "application/json");
            var response = _client.Send(request);
            using var reader = new StreamReader(response.Content.ReadAsStream());
            return reader.ReadToEnd();
        }
        public static void Main(string[] args)
        {
            // Login to NeoEdge Central by Admin User
            loginNeoEdgeCentral(_adminEmail, _adminPassword);

            // Read Roles, Projects List
            retrieveRoleList();
            retrieveProjectList();

            // Find User by Email
            //var user = findUserByEmail("abc@ecloudvalley.com");
            var user = findUserByEmail("kevin.kao@ecloudvalley.com");

            Console.WriteLine(user.name);
        }
    }
}




